﻿Imports System.Globalization
Imports System.Reflection
Imports Syncfusion.UI.Xaml.Charts
Imports sfChartTest
Imports System.Windows.Threading
Imports System.Collections.ObjectModel

Class MainWindow

    Dim TestColl As ViewModel
    Public BaseTicker As Ticker

    Public XAxis As CustomDateTimeCategoryXAxis

    Public YAxisPrice As CustomLogarithmicYAxis
    Public YAxisVolume1 As CustomNumericalYAxis
    Public YAxisVolume2 As CustomNumericalYAxis
    Public YAxisAnnotation As NumericalAxis

    Public SeriesPrice As New FastSteplineBitmapSeriesExt
    Dim SeriesVolume1 As New FastSteplineBitmapSeriesExt
    Dim SeriesVolume2 As New FastSteplineBitmapSeriesExt
    Dim SeriesAnnotation As New FastLineBitmapSeries

    Dim scrollBar As SfChartResizableBar
    Dim zoomDays As Integer

    Dim BaseBrush As New SolidColorBrush(Color.FromArgb(255, 100, 200, 255))



    Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        DataContext = Me

        TestColl = New ViewModel
        BaseTicker = TestColl.Tickers(0)

        Axes()
        Series()

        ClearAxes()

        AddRow0()
        AddRow1()
        AddRow2()
        AddAnnotationRow()


        'SET NO ANTIALIAS
        Dim image As Image = GetPrivateField(chart, "fastRenderDevice")
        If (Not (image) Is Nothing) Then
            RenderOptions.SetBitmapScalingMode(image, BitmapScalingMode.NearestNeighbor)
        End If

        ZoomX()

    End Sub

    Private Function GetPrivateField(ByVal chart As SfChart, ByVal name As String) As Image
        Dim info As FieldInfo = chart.GetType().GetField(name, BindingFlags.Instance Or (BindingFlags.NonPublic Or BindingFlags.Public))
        If (Not (info) Is Nothing) Then
            Return CType(info.GetValue(chart), Image)
        End If
        Return Nothing
    End Function


    Sub Axes()

        'PRIMARY AXIS
        XAxis = chart.PrimaryAxis
        XAxis.Name = NameOf(XAxis)
        XAxis.LabelFormat = "M/d/yy"
        XAxis.Interval = 1
        XAxis.MainWindow = Me

        customDateTimeCategoryXAxis1 = chart1.PrimaryAxis
        customDateTimeCategoryXAxis1.Name = NameOf(XAxis)
        customDateTimeCategoryXAxis1.LabelFormat = "M/d/yy"
        customDateTimeCategoryXAxis1.Interval = 1
        customDateTimeCategoryXAxis1.MainWindow = Me

        'Y AXIS PRICE
        YAxisPrice = New CustomLogarithmicYAxis
        YAxisPrice.Name = NameOf(YAxisPrice)
        YAxisPrice.LogarithmicBase = LogBase
        YAxisPrice.FontSize = 12
        YAxisPrice.ShowGridLines = False
        YAxisPrice.Header = "Price"
        YAxisPrice.MainWindow = Me

        'Y AXIS VOLUME 1
        YAxisVolume1 = New CustomNumericalYAxis
        YAxisVolume1.Name = NameOf(YAxisVolume1)
        YAxisVolume1.Minimum = 0
        YAxisVolume1.FontSize = 12
        YAxisVolume1.ShowGridLines = False
        YAxisVolume1.Visibility = Visibility.Visible
        YAxisVolume1.Header = "Volume 1"

        'Y AXIS VOLUME 2
        YAxisVolume2 = New CustomNumericalYAxis
        YAxisVolume2.Name = NameOf(YAxisVolume2)
        YAxisVolume2.Minimum = 0
        YAxisVolume2.FontSize = 12
        YAxisVolume2.ShowGridLines = False
        YAxisVolume2.Visibility = Visibility.Visible
        YAxisVolume2.Header = "Volume 2"

        'Y AXIS ANNOTATION
        YAxisAnnotation = New NumericalAxis
        With YAxisAnnotation
            .Name = NameOf(YAxisAnnotation)
            .ShowGridLines = False
            .Visibility = Visibility.Collapsed
        End With


    End Sub

    Sub Series()

        'SERIES PRICE
        With SeriesPrice
            .Name = NameOf(SeriesPrice)
            .XBindingPath = NameOf(Minute.DateTime)
            .YBindingPath = NameOf(Minute.Price)
            .Interior = New SolidColorBrush(Color.FromArgb(255, 0, 255, 0))
            .StrokeThickness = 1
            .YAxis = YAxisPrice
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart1.Series.Add(SeriesPrice)

        'SERIES VOLUME 1
        With SeriesVolume1
            .Name = NameOf(SeriesVolume1)
            .XBindingPath = NameOf(Minute.DateTime)
            .YBindingPath = NameOf(Minute.Volume1)
            .StrokeThickness = 1
            .Interior = BaseBrush
            .YAxis = YAxisVolume1
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesVolume1)

        'SERIES VOLUME 2
        With SeriesVolume2
            .Name = NameOf(SeriesVolume2)
            .XBindingPath = NameOf(Minute.DateTime)
            .YBindingPath = NameOf(Minute.Volume2)
            .StrokeThickness = 1
            .Interior = BaseBrush
            .YAxis = YAxisVolume2
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesVolume2)

        'SERIES ANNOTATION
        With SeriesAnnotation
            .YAxis = YAxisAnnotation
        End With
        chart.Series.Add(SeriesAnnotation)

    End Sub

    Sub ClearAxes()

        For i = chart.Axes.Count - 1 To 0 Step -1
            If Not chart.Axes(i) Is XAxis Then
                chart.Axes.RemoveAt(i)
            End If
        Next

    End Sub

    Sub AddRow0()

        chart.RowDefinitions.Add(New ChartRowDefinition With {.BorderThickness = 1, .BorderStroke = New SolidColorBrush(Colors.Red), .Height = 1})
        chart.Axes.Add(YAxisVolume2)
        SfChart.SetRow(YAxisVolume2, 0)

    End Sub

    Sub AddRow1()

        chart.RowDefinitions.Add(New ChartRowDefinition With {.BorderThickness = 1, .BorderStroke = New SolidColorBrush(Colors.Red), .Height = 1})
        chart.Axes.Add(YAxisVolume1)
        SfChart.SetRow(YAxisVolume1, 1)

    End Sub

    Sub AddRow2()

        ' chart.RowDefinitions.Add(New ChartRowDefinition With {.BorderThickness = 1, .BorderStroke = New SolidColorBrush(Colors.Red), .Height = 1})
        'chart1.Axes.Add(YAxisPrice)
        'SfChart.SetRow(YAxisPrice, 2)

    End Sub

    Sub AddAnnotationRow()

        SfChart.SetRow(YAxisAnnotation, 0) 'row starts from 0 at the bottom
        SfChart.SetRowSpan(YAxisAnnotation, 3) 'row ends to 3rd row at the top
        chart.Axes.Add(YAxisAnnotation)

    End Sub


    Private Sub btZoomAll_Click(sender As Object, e As RoutedEventArgs) Handles btZoomAll.Click

        zoomDays = 0
        ZoomX()

    End Sub

    Private Sub btZoom3days_Click(sender As Object, e As RoutedEventArgs) Handles btZoom3days.Click

        zoomDays = 3
        ZoomX()

    End Sub

    Private Sub btZoom2days_Click(sender As Object, e As RoutedEventArgs) Handles btZoom2days.Click

        zoomDays = 2
        ZoomX()

    End Sub

    Private Sub btZoom1day_Click(sender As Object, e As RoutedEventArgs) Handles btZoom1day.Click

        zoomDays = 1
        ZoomX()

    End Sub

    Sub ZoomX()

        If zoomDays > 0 Then

            XAxis.ZoomPosition = 1 - (zoomDays / BaseTicker.TotalDays)
            XAxis.ZoomFactor = zoomDays / BaseTicker.TotalDays

            XAxis.EnableScrollBar = True
            'scrollBar.RangeEnd = 1
            customDateTimeCategoryXAxis1.EnableScrollBar = True

            XAxis.IntervalType = DateTimeIntervalType.Days
            customDateTimeCategoryXAxis1.IntervalType = DateTimeIntervalType.Days
        Else 'zoom all

            XAxis.ZoomPosition = 0
            XAxis.ZoomFactor = 1
            XAxis.EnableScrollBar = False

            XAxis.IntervalType = DateTimeIntervalType.Months
            customDateTimeCategoryXAxis1.IntervalType = DateTimeIntervalType.Months

        End If

    End Sub

    Private Sub chart_Loaded(sender As Object, e As RoutedEventArgs) Handles chart.Loaded
        scrollBar = CType(chart.PrimaryAxis.GetType.GetField("sfChartResizableBar", BindingFlags.NonPublic Or BindingFlags.GetField Or BindingFlags.Instance).GetValue(chart.PrimaryAxis), SfChartResizableBar)
    End Sub

    Private Sub ActualRangeChanged(sender As Object, e As ActualRangeChangedEventArgs) Handles customDateTimeCategoryXAxis.ActualRangeChanged
        customDateTimeCategoryXAxis1.ZoomFactor = customDateTimeCategoryXAxis.ZoomFactor
        customDateTimeCategoryXAxis1.ZoomPosition = customDateTimeCategoryXAxis.ZoomPosition
    End Sub

    Private Sub ActualRangeChanged1(sender As Object, e As ActualRangeChangedEventArgs) Handles customDateTimeCategoryXAxis1.ActualRangeChanged
        customDateTimeCategoryXAxis.ZoomPosition = customDateTimeCategoryXAxis1.ZoomPosition
    End Sub

End Class

